function y = insigmat(fpml,r0,r,cstepml)
% INSIGMAT : computes the value(s) of sigma tilda involved in the PML at point(s) r

% VARIABLES IN :
% fpml : [handle function] the integrand involved in the definition of sigma tilda 
% r0 : [double] location of the PML inner side
% r : [array of double] point(s) where the function sigma tilda is evaluated
% cstepml : (double] constant involved in the definition of sigma tilda 
%
% VARIABLES OUT :
% y : [array of double] values of sigma tilda at points r
%
% Author: Stephane BALAC, IRMAR, CNRS, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 17 june 2019
% Program distributed under the CeCILL-C Public License

y=zeros(size(r));
if infolog('MATLAB')
    for i=1:length(r)
        y(i)=cstepml*integral(fpml,r0,r(i));
    end
else
    for i=1:length(r)
        y(i)=cstepml*quad(fpml,r0,r(i));
    end
end
