% RDISK : Display the results for a micro-disk cavity
% An internal program called by WGRIN for display purposes.
%
% Author: Stephane BALAC, IRMAR, CNRS, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 17 July 2020

prompt='[?] Value of the external boundary untill where the mode is displayed = ';
Ro = input(prompt) ;
while Ro<R
    disp(['[!] Enter a value larger than R = ',num2str(R)]);
    Ro = input(prompt) ;
end
tag=input('[?] Name of the figures backup files (if empty, no backup) : ','s');
Qo=round(Ro/dr);   % number of the node located at R2
ri=Rmin+[0:Qi]*dr; % nodes inside the cavity
ro=[Qi+1:Qo]*dr;   % nodes outside the cavity
%----------------------------------------------
% number of points used in the radial and angular direction for
% image display purposes
% These values can be increased for a high image quality, especially for
% large values of q=m, but memory requirement will be higher
if expert_mode==0
nbpa=100; % the image is sampled with nbpa points in the angular direction
nbpr=100; % the image is sampled with nbpr points in the radial direction
else
    disp('    Image display quality (2D display)')
    nbpa=input('[?] Number of sampling points in the polar direction = ');
    if isempty(nbpa), nbpa=100; end
    nbpr=input('[?] Number of sampling points in the radial direction = ');
    if isempty(nbpr), nbpr=100; end
end
% nbpr must be smaller than the number of discretization nodes
if nbpr>Qo, nbpr=Qo; end
pasr=round(Qo/nbpr);
pr=(1:pasr:Qo)*dr;
theta=[0:pi/nbpa:2*pi];
[TR,THETA]=meshgrid(pr,theta);
Eim=exp(1i*m*theta);
xc=R*cos(theta);
yc=R*sin(theta);
%---------------------------------------------
%
cntdis=1;
while cntdis==1
    j=input('[?] Index of the computed modes to be displayed = ');
    figure('units','normalized','outerposition',[0 0 1 1]) % full screen UI window
    kj=k(j);
    lambdaj=2*pi/kj;
    Uj=V(:,j); % current eigenvector
    Uj=Uj./max(abs(Uj));
    U1=Uj(1:Qi);
    iU1=imag(U1);
    rU1=real(U1);
    alpha=0.5*atan(0.5*(sum(rU1.^2)-sum(iU1.^2))/sum(rU1.*iU1) );
    U1=U1*exp(1i*alpha);
    U2=U1(Qi)*besselh(m,1,kj*ro)/besselh(m,1,kj*R);
    U=[0;U1;transpose(U2)];
    U=U./max(abs(U));
    %-----------------
    subplot(3,2,1)
    plot([ri,ro],abs(U),'-b','LineWidth',3);
    hold on
    plot([R,R],[0,1],'-.k','LineWidth',2); % position of the cavity boundary
    ylim([0,1])
    grid
    hold off
    if strcmp(tetm,'TE')
        title(['Radial variation of H_z for \lambda = ',...
            num2str(real(lambdaj)),' \mu-m (modulus)']);
    else
        title(['Radial variation of E_z for \lambda = ',...
            num2str(real(lambdaj)),' \mu-m (modulus)']);
        
    end
    xlabel('r'); ylabel('Mode (arb. unit)'); xlim([0,Ro]);
    text(1.05*R,1/2,'Vacuum')
    text(0.75*R,1/2,'Disk')
    %-----------------
    subplot(3,2,3)
    plot([ri,ro],real(U),'-m','LineWidth',2);
    hold on
    plot([R,R],[-1,1],'-.k','LineWidth',2); % position of the cavity boundary
    hold off
    grid
    ylim([-1,1])
    if strcmp(tetm,'TE')
        title(['Radial variation of H_z for \lambda = ',...
            num2str(real(lambdaj)),' \mu-m (real part)']);
    else
        title(['Radial variation of E_z for \lambda = ',...
            num2str(real(lambdaj)),' \mu-m (real part)']);
        
    end
    
    xlabel('r'); ylabel('(arb. unit)'); xlim([0,Ro]);
    text(1.05*R,1/2,'Vacuum')
    text(0.75*R,1/2,'Disk')
    %-----------------
    subplot(3,2,5)
    plot([ri,ro],imag(U),'-c','LineWidth',2);
    hold on
    plot([R,R],[-1,1],'-.k','LineWidth',2); % position of the cavity boundary
    hold off
    grid
    ylim([-1,1])
    if strcmp(tetm,'TE')
        title(['Radial variation of H_z for \lambda = ',...
            num2str(real(lambdaj)),' \mu-m (imaginary part)']);
    else
        title(['Radial variation of E_z for \lambda = ',...
            num2str(real(lambdaj)),' \mu-m (imaginary part)']);
        
    end
    xlabel('r'); ylabel('(arb. unit)'); xlim([0,Ro]);
    text(1.05*R,1/2,'Vaccum')
    text(0.75*R,1/2,'Disk')
    %-----------------
    Vj=transpose(U(1:pasr:Qo)*Eim);
    subplot(3,2,2)
    contourf(TR.*cos(THETA), TR.*sin(THETA), abs(Vj),'LineStyle','none');
    axis('equal'); colorbar
    xlim([-Ro,Ro]); ylim([-Ro,Ro]);
    xlabel('x'); ylabel('y');
    if strcmp(tetm,'TE')
        title('Modulus of H_z');
    else
        title('Modulus of E_z');
    end
    grid;
    hold on; plot(xc,yc,'.-k');
    %-----------------
    subplot(3,2,4)
    contourf(TR.*cos(THETA), TR.*sin(THETA), real(Vj),'LineStyle','none');
    axis('equal'); colorbar
    xlim([-Ro,Ro]); ylim([-Ro,Ro]);
    xlabel('x'); ylabel('y');
    if strcmp(tetm,'TE')
        title('Real part of H_z');
    else
        title('Real part of E_z');
    end
    grid;
    hold on; plot(xc,yc,'.-k');
    %-----------------
    subplot(3,2,6)
    contourf(TR.*cos(THETA), TR.*sin(THETA), imag(Vj),'LineStyle','none');
    axis('equal'); colorbar
    if strcmp(tetm,'TE')
        title('Imaginary part of H_z');
    else
        title('Imaginary part of E_z');
    end
    grid;
    xlim([-Ro,Ro]); ylim([-Ro,Ro]);
    xlabel('x'); ylabel('y');
    hold on; plot(xc,yc,'.-k');
  
    if isempty(tag)==0, print([tag,num2str(j)],'-dpng'); end
    cntdis=input('[?] Display another mode (y/n) ? ans =  : ','s');
    if cntdis=='y', cntdis=1; else, cntdis=0; end
end


