%RSPECTRUM DISPLAY THE SPECTRUM OF EIGENVALUE PROBLEM MATRIX FOR VARIOUS
%SET OF PML PARAMETERS. 
%THIS ALLOWS TO DISTINGUISH BETWEEN SPURIOUS RESONANCES AND TRUE
%RESONANCES (WGM RESONANCES AND OUTER RESONANCES).
%AT LEAST TWO SET OF PML PARAMETERS ARE REQUIRED.
% 
% To use RSPECTRUM, a first computation has to be made using the WGRIN
% program. After WGRIN completes, RSPECTRUM can be launched.
% In order to clearly identify the spurious resonances it is recommended to
% change the value of sigma0 preferably. A modification of the regularity
% parameter qpml do not imply generally a shift large enough to cleary
% identify the spurious resonances.
%
% Author: Stephane BALAC, IRMAR, CNRS, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 28 July 2020
% 

clear TK
TK(1,:)=k;
ind=1;
cntdis=1;
while cntdis==1
    ind=ind+1;
    disp('[?] Select the PML parameter to modify :')
    npml=input('    (1) R1   (2) sigma0   (3) qpml   : rep = ');
    switch npml
        case 1
            old_R1=R1;
            R1=input('[?] New value of the PML end : R1 = ');
            Q=round(R1/dr);   % to be sure to have a node at the interface
            R1=Q*dr;         % slight modification of the PML location
            r=Rmin+[1:Q-1]*dr; % nodes position
        case 2
            old_sigma0=sigma0;
            sigma0=input('[?] New value of the PML amplification coefficient : sigma0 = ');
        case 3
            old_qpml=qpml;
            qpml=input('[?] New value of the PML regularity parameter : qpml = ');
        otherwise
            warning('Sorry, wrong answer ...')
            return
    end
    disp(['[*] PML parameters : R1 = ',num2str(R1),', sigma0 = ',num2str(sigma0),', qpml = ',num2str(qpml)]);
    %------------------------------
    % Definition of the new PML
    %------------------------------
    fpml=@(t) (t-R0).^qpml.*(R1-t).^qpml; % fonction involved in the definition of the PML
    if infolog('MATLAB')
        cstepml=sigma0/integral(fpml,R0,R1);
    else
        cstepml=sigma0/quad(fpml,R0,R1);
    end
    % Complex shift variable
    sigmat=@(r) insigmat(fpml,R0,r,cstepml); % function sigma tilda involved in the def. of the PML
    sigma=@(r) sigmat(r)+r.*cstepml.*fpml(r); % function sigma involved in the def. of the PML
    pmld=@(r) ones(size(r))+1i*sigma(r); % function d involved in the def. of the PML
    pmldt=@(r) ones(size(r))+1i*sigmat(r); % function d tilda involved in the def. of the PML
    
    % ------------------------------
    % Computation of the resonances
    % -------------------------------
    % Computation of the matrix elements
    t=cputime;
    disp(' ');
    disp('[*] Computation of the FDM matrices for the new set of PML parameters');
    [A,C]=matrice(Qi,Q,q,p,d,R,r,dr,N,pmld,pmldt);
    % Computation of the eigen-elements of the matrix
    disp('[*] Solving the generalized eigenvalue problem ');
    shift=(2*pi/lambdar)^2;
    [V,D,flag] = eigs(A,C,nbres,shift);
    K=diag(D);
    knew=sqrt(K);
    % the resonances are sorted according to their proximity to the target
    % resonance value kr=2*pi/lambdar;
    [void,tabJo] = sort(abs(2*pi./knew-lambdar));
    knew=knew(tabJo);
    clear void
    TK(ind,:)=knew;
    % we set the PML parameters to their initial values
    switch npml
        case 1
            R1=old_R1;
        case 2
            sigma0=old_sigma0;
        case 3
            qpml=old_qpml;
    end
    cntdis=input('[?] Modify another PML parameter (y/n) ? ans =  : ','s');
    if cntdis=='y', cntdis=1; else, cntdis=0; end
end

% ----------------------------------------------------
% Display the spectrum for the different computations
% ----------------------------------------------------

tabsymb=['xb';'or';'pm';'>g'];
figure
hold on
for ind=1:size(TK,1)
    plot(real(TK(ind,:)),imag(TK(ind,:)),tabsymb(mod(ind,size(tabsymb,1)),:))
end
plot(2*pi/lambdar,0,'*k')
xlabel('Re(k) [\mu-m^{-1}] '), ylabel('Im(k) [\mu-m^{-1}]');
%ylim([-5,0]), xlim([0,20])
grid

figure
hold on
for ind=1:size(TK,1)
    plot(real(2*pi./TK(ind,:)),imag(2*pi./TK(ind,:)),tabsymb(mod(ind,size(tabsymb,1)),:))
end
plot(lambdar,0,'*k')
xlabel('Re(\lambda) [\mu-m]'), ylabel('Im(\lambda) [\mu-m]');
%ylim([0,0.25]), xlim([0 1.5])
grid


