%RSPURIOUS DISPLAY THE EIGENVECTORS IN THE CAVITY AND IN THE OUTER PML DOMAIN
%ALLOWS TO IDENTIFY SPURIOUS MODES
% 
% To use RSPURIOUS, a first computation has to be made using the WGRIN
% program. After WGRIN completes, RSPURIOUS can be launched.
%
% Author: Stephane BALAC, IRMAR, CNRS, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 28 July 2020
% 


cntdis=1;
while cntdis==1
    j=input('[?] Index of the computed modes to be displayed = ');
    figure('units','normalized','outerposition',[0 0 1 1]) % full screen UI window
    kj=k(j);
    lambdaj=2*pi/kj;
    Uj=V(:,j); % current eigenvector
    U=Uj./max(abs(Uj));
    %-----------------
    subplot(3,1,1)
    plot(r,abs(U),'-b','LineWidth',3);
    hold on
    plot([R,R],[0,1],'-.k','LineWidth',2); % position of the cavity boundary
    ylim([0,1])
    grid
    hold off
    title(['Radial variation of the eigenvector u for \lambda = ',...
        num2str(real(lambdaj)),' \mu-m (modulus)']);
    xlabel('r'); ylabel('Mode (arb. unit)'); xlim([0,R1]);
    text(1.05*R,1/2,'PML')
    text(0.75*R,1/2,'Disk')
    %-----------------
    subplot(3,1,2)
    plot(r,real(U),'-m','LineWidth',2);
    hold on
    plot([R,R],[-1,1],'-.k','LineWidth',2); % position of the cavity boundary
    hold off
    grid
    ylim([-1,1])
    title(['Radial variation of the eigenvector u for \lambda = ',...
        num2str(real(lambdaj)),' \mu-m (real part)']);
    xlabel('r'); ylabel('(arb. unit)'); xlim([0,R1]);
    text(1.05*R,1/2,'PML')
    text(0.75*R,1/2,'Disk')
    %-----------------
    subplot(3,1,3)
    plot(r,imag(U),'-c','LineWidth',2);
    hold on
    plot([R,R],[-1,1],'-.k','LineWidth',2); % position of the cavity boundary
    hold off
    grid
    ylim([-1,1])
    
    title(['Radial variation of the eigenvector u for \lambda = ',...
        num2str(real(lambdaj)),' \mu-m (imaginary part)']);
    xlabel('r'); ylabel('(arb. unit)'); xlim([0,R1]);
    text(1.05*R,1/2,'PML')
    text(0.75*R,1/2,'Disk')
    
    cntdis=input('[?] Display another mode (y/n) ? ans =  : ','s');
    if cntdis=='y', cntdis=1; else, cntdis=0; end
end

