%WGRIN COMPUTES THE RESONANCE WAVELENGTHS OF A GRADED INDEX OPTICAL MICRO-DISK 
%   OR MICRO-SPHERE
%
% Author: Stephane BALAC, IRMAR, CNRS, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 28 June 2019
%
% Copyright or © or Copr. Stephane BALAC (01 June 2019)
% 
% This software is governed by the CeCILL-C license under French law 
% and abiding by the rules of distribution of free software.
% You can  use, modify and/ or redistribute the software under the terms of
% the CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
% URL "http://www.cecill.info". 
% 
% As a counterpart to the access to the source code and  rights to copy,
% modify and redistribute granted by the license, users are provided only
% with a limited warranty  and the software's author,  the holder of the
% economic rights,  and the successive licensors  have only  limited
% liability. 
% 
% In this respect, the user's attention is drawn to the risks associated
% with loading,  using,  modifying and/or developing or reproducing the
% software by the user in light of its specific status of free software,
% that may mean  that it is complicated to manipulate,  and  that  also
% therefore means  that it is reserved for developers  and  experienced
% professionals having in-depth computer knowledge. Users are therefore
% encouraged to load and test the software's suitability as regards their
% requirements in conditions enabling the security of their systems and/or 
% data to be ensured and,  more generally, to use and operate it in the 
% same conditions as regards security. 
% 
% The fact that you are presently reading this means that you have had
% knowledge of the CeCILL-C license and that you accept its terms.

clear; close all;
format long;
%
% ----------------------
% Expert mode selector :
% set the variable expert_mode to 1 for using the programm in expert mode
% and to 0 to use default parameters values.
expert_mode=0;
% ---------------------
%
prompt = '[?] Geometry : sphere (0) or disk (1) : ans = ';
d = input(prompt) ;
while d~=0 &&  d~=1
    d = input(prompt) ;
end
R=input('[?] Radius in mu-m: R = ');
n0=input('[?] Optical index at the boundary: n0 = ');

if d==0 % case of a sphere 
    ell=input('[?] Polar mode index: ell = ');
    q=sqrt(ell*(ell+1)); % q^2=ell*(ell+1)
 else % case of a disk
    m=input('[?] Polar mode index: m = '); 
    q=m;
end
tetm=input('[?] TE or TM  mode : ','s');
if strcmp(tetm,'TE')||strcmp(tetm,'te')
    tetm='TE';
    disp('[*] TE modes');
    if d==1 % case of a disk
        p=-1;
    else % case of a sphere
        p=1;
    end
else
    tetm='TM';
    disp('[*] TM modes');
    if d==1 % case of a disk
        p=1;
    else % case of a sphere
        p=-1;
    end
end
nbres=input('[?] Number of resonances to compute = ');
if expert_mode==1
    lambdat=input('[?] Target resonance wavelength in mu-m : lambda = ');
    if isempty(lambdat), lambdat=2*pi*R*n0/q; end
else
    lambdat=2*pi*R*n0/q;
end
%
optindex; % definition of the radially varying optical index and its derivative
%
% Plot of the radially varying optical index
%
figure
r=linspace(0,R,200);
plot(r,N(r),'-b','LineWidth',2);
hold on
r=linspace(1.0001*R,1.5*R,100);
plot(r,N(r),'-b','LineWidth',2)
grid; xlim([0,1.5*R]);
% ylim([0,2*n0]);
xlabel('r')
title('Optical index profile')
r=[];% 
% ----------------------------------
% Finite Difference discretization
% ----------------------------------
Rmin=0*R;    % for inner truncation of the domain purposes
if expert_mode==1
    Qi=input('[?] Number of discretization sub-intervals inside the cavity: Qi = ');
    if isempty(Qi),Qi=1E4;end
else
    Qi=1E4;
end
dr=(R-Rmin)/Qi;
if expert_mode==1
   disp(['[*] Step-size of the FDM = ', num2str(dr)]);
end
% ---------------------
% Definition of the PML
% ---------------------
% parameters of the PML
if expert_mode==0
    R1=2*R;  % PML outer boundary position
    sigma0=16*log(10)*n0*R/(q*R1);
    qpml=2; % regularity index for the PML (integer value)
else
    R1=input('[?] PML outer boundary position = ');
    if isempty(R1),R1=2*R;end
    sigma0=input('[?] PML parameter sigma0 = ');
    if isempty(sigma0),sigma0=16*log(10)*n0*R/(q*R1);end
    qpml=input('[?] PML regularity = ');
    if isempty(qpml),qpml=2;end
end
% The position of the PML is adjusted to ensure that a node coincides
% with the PML  end
R0=R; % PML entry location
Q=round(R1/dr);   % to be sure to have a node at the interface
R1=Q*dr;         % slight modification of the PML location
r=Rmin+[1:Q-1]*dr; % nodes position
disp(['[*] PML parameters : R1 = ',num2str(R1),', sigma0 = ',num2str(sigma0),', qpml = ',num2str(qpml)]);
% 
fpml=@(t) (t-R0).^qpml.*(R1-t).^qpml; % fonction involved in the definition of the PML
% Note that the Matlab function integral is not (yet?) implemented under Octave
if infolog('MATLAB')
    cstepml=sigma0/integral(fpml,R0,R1);
else
    cstepml=sigma0/quad(fpml,R0,R1);
end
% Complex shift variable 
sigmat=@(r) insigmat(fpml,R0,r,cstepml); % function sigma tilda involved in the def. of the PML
sigma=@(r) sigmat(r)+r.*cstepml.*fpml(r); % function sigma involved in the def. of the PML
pmld=@(r) ones(size(r))+1i*sigma(r); % function d involved in the def. of the PML
pmldt=@(r) ones(size(r))+1i*sigmat(r); % function d tilda involved in the def. of the PML

% ------------------------------
% Computation of the resonances
% -------------------------------
% Computation of the matrix elements
t=cputime;
disp(' ');
disp('[*] Computation of the FDM matrices ');
[A,C]=matrice(Qi,Q,q,p,d,R,r,dr,N,pmld,pmldt);
% Computation of the eigen-elements of the matrix
disp('[*] Solving the generalized eigenvalue problem ');
shift=(2*pi/lambdat)^2;
[V,D,flag] = eigs(A,C,nbres,shift);
K=diag(D);
k=sqrt(K);
% the resonances are sorted according to their proximity to the target
% resonance value
%kr=2*pi/lambdat;
[void,tabJo] = sort(abs(2*pi./k-lambdat));
k=k(tabJo);
clear void
void=V;
V=void(:,tabJo);
% ---------------
% Results display
% ---------------
disp(['[*] CPU time for the computations (sec.): ',num2str(cputime-t)]);
disp(' ');
for j=1:length(k)
    kj=k(j);
    lambdaj=2*pi/kj;
    disp(['Wavenumber #',num2str(j),' = ',num2str(kj,16),' mu-m^(-1)']);
    disp(['Wavelength #',num2str(j),' = ',num2str(lambdaj,16),' mu-m']);
    disp(' ');
end
%
anspur=input('[?] Auto-detection of spurious modes (y/n) ? ans = ','s');
if anspur=='y'
    jdel=[];
    jkeep=[];
    for j=1:length(k)
        Uj=V(:,j); % current eigenvector
        Uj=Uj./max(abs(Uj));
        Uj1=Uj(1:Qi);
        Uj2=Uj(Qi+1:Q-1);
        if norm(Uj2) > norm(Uj1) % we have a spurious mode located in the PML
            jdel=[jdel,j]; % index to remove
        else
            jkeep=[jkeep,j];
        end
    end
    kres=k(jkeep); % The true resonances
    disp('[*] The resonances identified as WGM are :')
    for j=1:length(kres)
        kj=kres(j);
        lambdaj=2*pi/kj;
        disp(['Wavenumber #',num2str(jkeep(j)),' = ',num2str(kj,16),' mu-m^(-1)']);
        disp(['Wavelength #',num2str(jkeep(j)),' = ',num2str(lambdaj,16),' mu-m']);
        disp(' ');
    end
end
% Graphical display
answ=input('[?] Graphical display of the modes (y/n) ? ans = ','s');
if answ~='y'
    return;
else
    if d==1 
        rdisk;
    else
        rsphere;
    end
end
